@@ -18,7 +18,6 @@ import re
 import time
 
 from oslo_utils import strutils
-import six
 from tempest.lib.cli import base
 from tempest.lib.cli import output_parser
 from tempest.lib.common.utils import data_utils
@@ -48,7 +47,7 @@ def not_found_wrapper(f):
         except tempest_lib_exc.CommandFailed as e:
             for regexp in (r'No (\w+) with a name or ID',
                            r'not(.*){0,5}found'):
-                if re.search(regexp, six.text_type(e.stderr)):
+                if re.search(regexp, str(e.stderr)):
                     # Raise appropriate 'NotFound' error
                     raise tempest_lib_exc.NotFound()
             raise
@@ -62,7 +61,7 @@ def forbidden_wrapper(f):
         try:
             return f(self, *args, **kwargs)
         except tempest_lib_exc.CommandFailed as e:
-            if re.search('HTTP 403', six.text_type(e.stderr)):
+            if re.search('HTTP 403', str(e.stderr)):
                 # Raise appropriate 'Forbidden' error.
                 raise tempest_lib_exc.Forbidden()
             raise
@@ -201,10 +200,10 @@ class ManilaCLIClient(base.CLIClient):
         if name is None:
             name = data_utils.rand_name('manilaclient_functional_test')
         dhss = driver_handles_share_servers
-        if not isinstance(dhss, six.string_types):
-            dhss = six.text_type(dhss)
-        if not isinstance(is_public, six.string_types):
-            is_public = six.text_type(is_public)
+        if not isinstance(dhss, str):
+            dhss = str(dhss)
+        if not isinstance(is_public, str):
+            is_public = str(is_public)
 
         cmd = ('type-create %(name)s %(dhss)s --is-public %(is_public)s ') % {
             'name': name, 'dhss': dhss, 'is_public': is_public}
@@ -213,34 +212,31 @@ class ManilaCLIClient(base.CLIClient):
             cmd += " --description " + description
 
         if snapshot_support is not None:
-            if not isinstance(snapshot_support, six.string_types):
-                snapshot_support = six.text_type(snapshot_support)
+            if not isinstance(snapshot_support, str):
+                snapshot_support = str(snapshot_support)
             cmd += " --snapshot-support " + snapshot_support
 
         if create_share_from_snapshot is not None:
-            if not isinstance(create_share_from_snapshot, six.string_types):
-                create_share_from_snapshot = six.text_type(
-                    create_share_from_snapshot)
+            if not isinstance(create_share_from_snapshot, str):
+                create_share_from_snapshot = str(create_share_from_snapshot)
             cmd += (" --create-share-from-snapshot-support " +
                     create_share_from_snapshot)
 
         if revert_to_snapshot is not None:
-            if not isinstance(revert_to_snapshot, six.string_types):
-                revert_to_snapshot = six.text_type(
-                    revert_to_snapshot)
+            if not isinstance(revert_to_snapshot, str):
+                revert_to_snapshot = str(revert_to_snapshot)
             cmd += (" --revert-to-snapshot-support " + revert_to_snapshot)
 
         if mount_snapshot is not None:
-            if not isinstance(mount_snapshot, six.string_types):
-                mount_snapshot = six.text_type(
-                    mount_snapshot)
+            if not isinstance(mount_snapshot, str):
+                mount_snapshot = str(mount_snapshot)
             cmd += (" --mount-snapshot-support " + mount_snapshot)
 
         if extra_specs is not None:
             extra_spec_str = ''
             for k, v in extra_specs.items():
-                if not isinstance(v, six.string_types):
-                    extra_specs[k] = six.text_type(v)
+                if not isinstance(v, str):
+                    extra_specs[k] = str(v)
                 extra_spec_str += "{}='{}' ".format(k, v)
             cmd += " --extra_specs " + extra_spec_str
 
@@ -267,8 +263,8 @@ class ManilaCLIClient(base.CLIClient):
             'share_type_id': share_type_id}
 
         if is_public is not None:
-            if not isinstance(is_public, six.string_types):
-                is_public = six.text_type(is_public)
+            if not isinstance(is_public, str):
+                is_public = str(is_public)
             cmd += " --is_public " + is_public
 
         if description:
@@ -551,7 +547,7 @@ class ManilaCLIClient(base.CLIClient):
 
     @staticmethod
     def _stranslate_to_cli_optional_param(param):
-        if len(param) < 1 or not isinstance(param, six.string_types):
+        if len(param) < 1 or not isinstance(param, str):
             raise exceptions.InvalidData(
                 'Provided wrong parameter for translation.')
         while not param[0:2] == '--':
@@ -587,6 +583,71 @@ class ManilaCLIClient(base.CLIClient):
         share_networks = utils.listing(share_networks_raw)
         return share_networks
 
+    def share_network_reset_state(self, id=None, state=None,
+                                  microversion=None):
+        cmd = 'share-network-reset-state %s ' % id
+        if state:
+            cmd += '--state %s' % state
+        share_network_raw = self.manila(cmd, microversion=microversion)
+        share_network = utils.listing(share_network_raw)
+        return share_network
+
+    def share_network_security_service_add(
+            self, share_network_id, security_service_id, microversion=None):
+        cmd = ('share-network-security-service-add %(network_id)s '
+               '%(service_id)s' % {'network_id': share_network_id,
+                                   'service_id': security_service_id})
+        self.manila(cmd, microversion=microversion)
+
+    def share_network_security_service_update(
+            self, share_network_id, current_security_service_id,
+            new_security_service_id, microversion=None):
+        cmd = (
+            'share-network-security-service-update %(network_id)s '
+            '%(current_service_id)s %(new_security_service_id)s' % {
+                'network_id': share_network_id,
+                'current_service_id': current_security_service_id,
+                'new_security_service_id': new_security_service_id})
+        self.manila(cmd, microversion=microversion)
+
+    def share_network_security_service_add_check(
+            self, share_network_id, security_service_id,
+            reset=False, microversion=None):
+        cmd = (
+            'share-network-security-service-add-check %(network_id)s '
+            '%(security_service_id)s' % {
+                'network_id': share_network_id,
+                'security_service_id': security_service_id})
+
+        if reset:
+            cmd += '--reset %s' % reset
+
+        return output_parser.details(
+            self.manila(cmd, microversion=microversion))
+
+    def share_network_security_service_update_check(
+            self, share_network_id, current_security_service_id,
+            new_security_service_id, reset=False, microversion=None):
+        cmd = (
+            'share-network-security-service-update-check %(network_id)s '
+            '%(current_security_service_id)s %(new_security_service_id)s ' % {
+                'network_id': share_network_id,
+                'current_security_service_id': current_security_service_id,
+                'new_security_service_id': new_security_service_id})
+
+        if reset:
+            cmd += '--reset %s' % reset
+
+        return output_parser.details(
+            self.manila(cmd, microversion=microversion))
+
+    def share_network_security_service_list(
+            self, share_network_id, microversion=None):
+        cmd = ('share-network-security-service-list %s' % share_network_id)
+        share_networks_raw = self.manila(cmd, microversion=microversion)
+        network_services = utils.listing(share_networks_raw)
+        return network_services
+
     def is_share_network_deleted(self, share_network, microversion=None):
         """Says whether share network is deleted or not.
 
@@ -608,6 +669,26 @@ class ManilaCLIClient(base.CLIClient):
             SHARE_NETWORK, res_id=share_network, interval=2, timeout=6,
             microversion=microversion)
 
+    def share_network_subnet_create_check(
+            self, share_network_id, neutron_net_id=None,
+            neutron_subnet_id=None, availability_zone=None, reset=False,
+            microversion=None):
+        params = self._combine_share_network_subnet_data(
+            neutron_net_id=neutron_net_id,
+            neutron_subnet_id=neutron_subnet_id,
+            availability_zone=availability_zone)
+        cmd = (
+            'share-network-subnet-create-check %(network_id)s '
+            '%(params)s' % {
+                'network_id': share_network_id,
+                'params': params})
+
+        if reset:
+            cmd += '--reset %s' % reset
+
+        return output_parser.details(
+            self.manila(cmd, microversion=microversion))
+
     # Share Network Subnets
 
     def _combine_share_network_subnet_data(self, neutron_net_id=None,
@@ -712,7 +793,7 @@ class ManilaCLIClient(base.CLIClient):
 
     def create_share(self, share_protocol, size, share_network=None,
                      share_type=None, name=None, description=None,
-                     public=False, snapshot=None, metadata=None,
+                     public=False, snapshot=None, metadata=None, wait=False,
                      microversion=None):
         """Creates a share.
 
@@ -726,6 +807,7 @@ class ManilaCLIClient(base.CLIClient):
             Default is False.
         :param snapshot: str -- Name or ID of a snapshot to use as source.
         :param metadata: dict -- key-value data to provide with share creation.
+        :param wait: bool - the client must wait for "available" state
         :param microversion: str -- API microversion that should be used.
         """
         cmd = 'create %(share_protocol)s %(size)s ' % {
@@ -741,7 +823,7 @@ class ManilaCLIClient(base.CLIClient):
             description = data_utils.rand_name('autotest_share_description')
         cmd += '--description %s ' % description
         if public:
-            cmd += '--public'
+            cmd += '--public '
         if snapshot is not None:
             cmd += '--snapshot %s ' % snapshot
         if metadata:
@@ -750,6 +832,8 @@ class ManilaCLIClient(base.CLIClient):
                 metadata_cli += '%(k)s=%(v)s ' % {'k': k, 'v': v}
             if metadata_cli:
                 cmd += '--metadata %s ' % metadata_cli
+        if wait:
+            cmd += '--wait '
         share_raw = self.manila(cmd, microversion=microversion)
         share = output_parser.details(share_raw)
         return share
@@ -784,25 +868,66 @@ class ManilaCLIClient(base.CLIClient):
 
     @not_found_wrapper
     @forbidden_wrapper
-    def delete_share(self, shares, microversion=None):
+    def delete_share(self, shares, share_group_id=None, wait=False,
+                     microversion=None):
         """Deletes share[s] by Names or IDs.
 
         :param shares: either str or list of str that can be either Name
             or ID of a share(s) that should be deleted.
+        :param share_group_id: a common share group ID for the shares being
+          deleted
+        :param wait: bool -- whether to wait for the shares to be deleted
         """
         if not isinstance(shares, list):
             shares = [shares]
         cmd = 'delete '
         for share in shares:
             cmd += '%s ' % share
+        if share_group_id:
+            cmd += '--share-group-id %s ' % share_group_id
+        if wait:
+            cmd += '--wait '
         return self.manila(cmd, microversion=microversion)
 
-    def list_shares(self, all_tenants=False, filters=None, columns=None,
-                    is_public=False, microversion=None):
+    @not_found_wrapper
+    @forbidden_wrapper
+    def soft_delete_share(self, shares, microversion=None):
+        """Soft Delete share[s] by Names or IDs.
+
+        :param shares: either str or list of str that can be either Name
+            or ID of a share(s) that should be soft deleted.
+        """
+        if not isinstance(shares, list):
+            shares = [shares]
+        cmd = 'soft-delete '
+        for share in shares:
+            cmd += '%s ' % share
+        return self.manila(cmd, microversion=microversion)
+
+    @not_found_wrapper
+    @forbidden_wrapper
+    def restore_share(self, shares, microversion=None):
+        """Restore share[s] by Names or IDs.
+
+        :param shares: either str or list of str that can be either Name
+            or ID of a share(s) that should be soft deleted.
+        """
+        if not isinstance(shares, list):
+            shares = [shares]
+        cmd = 'restore '
+        for share in shares:
+            cmd += '%s ' % share
+        return self.manila(cmd, microversion=microversion)
+
+    def list_shares(self, all_tenants=False, is_soft_deleted=False,
+                    filters=None, columns=None, is_public=False,
+                    microversion=None):
         """List shares.
 
         :param all_tenants: bool -- whether to list shares that belong
             only to current project or for all tenants.
+        :param is_soft_deleted: bool -- whether to list shares that has
+            been soft deleted to recycle bin.
         :param filters: dict -- filters for listing of shares.
             Example, input:
                 {'project_id': 'foo'}
@@ -814,12 +939,15 @@ class ManilaCLIClient(base.CLIClient):
             Example, "--columns Name,Size"
         :param is_public: bool -- should list public shares or not.
             Default is False.
+        :param microversion: str -- the request api version.
         """
         cmd = 'list '
         if all_tenants:
             cmd += '--all-tenants '
         if is_public:
             cmd += '--public '
+        if is_soft_deleted:
+            cmd += '--soft-deleted '
         if filters and isinstance(filters, dict):
             for k, v in filters.items():
                 cmd += '%(k)s=%(v)s ' % {
@@ -875,6 +1003,46 @@ class ManilaCLIClient(base.CLIClient):
             SHARE, res_id=share, interval=5, timeout=300,
             microversion=microversion)
 
+    def wait_for_share_soft_deletion(self, share_id, microversion=None):
+        body = self.get_share(share_id, microversion=microversion)
+        is_soft_deleted = body['is_soft_deleted']
+        start = int(time.time())
+
+        while is_soft_deleted == "False":
+            time.sleep(self.build_interval)
+            body = self.get_share(share_id, microversion=microversion)
+            is_soft_deleted = body['is_soft_deleted']
+
+            if is_soft_deleted == "True":
+                return
+
+            if int(time.time()) - start >= self.build_timeout:
+                message = ("Share %(share_id)s failed to be soft deleted "
+                           "within the required time %(build_timeout)s." %
+                           {"share_id": share_id,
+                            "build_timeout": self.build_timeout})
+                raise tempest_lib_exc.TimeoutException(message)
+
+    def wait_for_share_restore(self, share_id, microversion=None):
+        body = self.get_share(share_id, microversion=microversion)
+        is_soft_deleted = body['is_soft_deleted']
+        start = int(time.time())
+
+        while is_soft_deleted == "True":
+            time.sleep(self.build_interval)
+            body = self.get_share(share_id, microversion=microversion)
+            is_soft_deleted = body['is_soft_deleted']
+
+            if is_soft_deleted == "False":
+                return
+
+            if int(time.time()) - start >= self.build_timeout:
+                message = ("Share %(share_id)s failed to be restored "
+                           "within the required time %(build_timeout)s." %
+                           {"share_id": share_id,
+                            "build_timeout": self.build_timeout})
+                raise tempest_lib_exc.TimeoutException(message)
+
     def wait_for_resource_status(self, resource_id, status, microversion=None,
                                  resource_type="share"):
         """Waits for a share to reach a given status."""
@@ -926,7 +1094,7 @@ class ManilaCLIClient(base.CLIClient):
                                'dest': dest_host,
                                'share_id': share['id'],
                                'timeout': self.build_timeout,
-                               'status': six.text_type(statuses),
+                               'status': str(statuses),
                            })
                 raise tempest_lib_exc.TimeoutException(message)
         return share
@@ -1686,7 +1854,7 @@ class ManilaCLIClient(base.CLIClient):
                                'dest': dest_host,
                                'share_server_id': server['id'],
                                'timeout': self.build_timeout,
-                               'status': six.text_type(statuses),
+                               'status': str(statuses),
                            })
                 raise tempest_lib_exc.TimeoutException(message)
 
@@ -1759,12 +1927,18 @@ class ManilaCLIClient(base.CLIClient):
 
     # Share replicas
 
-    def create_share_replica(self, share, microversion=None):
+    def create_share_replica(self, share, availability_zone=None,
+                             share_network=None, microversion=None):
         """Create a share replica.
 
         :param share: str -- Name or ID of a share to create a replica of
         """
         cmd = "share-replica-create %s" % share
+        if availability_zone is not None:
+            cmd += " --availability_zone " + availability_zone
+        if share_network is not None:
+            cmd += " --share_network " + share_network
+
         replica = self.manila(cmd, microversion=microversion)
         return output_parser.details(replica)
 
