@@ -17,7 +17,7 @@ from unittest import mock
 
 import ddt
 
-from manilaclient.common.apiclient import base as common_base
+from manilaclient import base
 from manilaclient.common import constants
 from manilaclient.tests.unit import utils
 from manilaclient.tests.unit.v2 import fakes
@@ -143,7 +143,7 @@ class ShareServerManagerTest(utils.TestCase):
 
         with mock.patch.object(self.manager.api.client, 'post',
                                mock.Mock(return_value='fake')):
-            self.mock_object(common_base, 'getid',
+            self.mock_object(base, 'getid',
                              mock.Mock(return_value=share_server['id']))
             result = self.manager._action(action, share_server, info)
             self.manager.api.client.post.assert_called_once_with(
