@@ -10,7 +10,7 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
-import six
+from urllib import parse
 
 
 class HookableMixin(object):
@@ -44,28 +44,25 @@ def safe_issubclass(*args):
 
 
 def get_function_name(func):
-    if six.PY2:
-        if hasattr(func, "im_class"):
-            return "%s.%s" % (func.im_class, func.__name__)
-        else:
-            return "%s.%s" % (func.__module__, func.__name__)
-    else:
-        return "%s.%s" % (func.__module__, func.__qualname__)
+    return "%s.%s" % (func.__module__, func.__qualname__)
 
 
-def _encode(src):
-    """remove extra 'u' in PY2."""
-    if six.PY2 and isinstance(src, unicode):
-        return src.encode('utf-8')
-    return src
+def safe_urlencode(params_dict):
+    """Workaround incompatible change to urllib.parse
 
+    urllib's parse library used to adhere to RFC 2396 until
+    python 3.7. The library moved from RFC 2396 to RFC 3986
+    for quoting URL strings in python 3.7 and '~' is now
+    included in the set of reserved characters. [1]
 
-def unicode_key_value_to_string(src):
-    """Recursively converts dictionary keys to strings."""
-    if isinstance(src, dict):
-        return dict((_encode(k),
-                    _encode(unicode_key_value_to_string(v)))
-                    for k, v in src.items())
-    if isinstance(src, list):
-        return [unicode_key_value_to_string(l) for l in src]
-    return _encode(src)
+    This utility ensures "~" is never encoded.
+
+    See LP 1785283 [2] for more details.
+    [1] https://docs.python.org/3/library/urllib.parse.html#url-quoting
+    [2] https://bugs.launchpad.net/python-manilaclient/+bug/1785283
+
+    :param params_dict can be a list of (k,v) tuples, or a dictionary
+    """
+
+    parsed_params = parse.urlencode(params_dict)
+    return parsed_params.replace("%7E", "~")
