@@ -18,18 +18,16 @@ import collections
 import ipaddress
 from oslo_utils import uuidutils
 import re
-import six
 import string
 
 from manilaclient import api_versions
 from manilaclient import base
-from manilaclient.common.apiclient import base as common_base
 from manilaclient.common import constants
 from manilaclient import exceptions
 from manilaclient.v2 import share_instances
 
 
-class Share(common_base.Resource):
+class Share(base.Resource):
     """A share is an extra block level storage to the OpenStack instances."""
     def __repr__(self):
         return "<Share: %s>" % self.id
@@ -97,9 +95,9 @@ class Share(common_base.Resource):
         """Update the share with the provided state."""
         self.manager.reset_state(self, state)
 
-    def extend(self, new_size):
+    def extend(self, new_size, **kwargs):
         """Extend the size of the specified share."""
-        self.manager.extend(self, new_size)
+        self.manager.extend(self, new_size, **kwargs)
 
     def shrink(self, new_size):
         """Shrink the size of the specified share."""
@@ -113,6 +111,14 @@ class Share(common_base.Resource):
         """Reverts a share (in place) to a snapshot."""
         self.manager.revert_to_snapshot(self, snapshot)
 
+    def soft_delete(self):
+        """Soft delete a share to recycle bin"""
+        self.manager.soft_delete(self)
+
+    def restore(self):
+        """Restore a share from recycle bin"""
+        self.manager.restore(self)
+
 
 class ShareManager(base.ManagerWithFind):
     """Manage :class:`Share` resources."""
@@ -121,7 +127,7 @@ class ShareManager(base.ManagerWithFind):
     def create(self, share_proto, size, snapshot_id=None, name=None,
                description=None, metadata=None, share_network=None,
                share_type=None, is_public=False, availability_zone=None,
-               share_group_id=None):
+               share_group_id=None, scheduler_hints=None):
         """Create a share.
 
         :param share_proto: text - share protocol for new share available
@@ -136,9 +142,14 @@ class ShareManager(base.ManagerWithFind):
         :param is_public: bool, whether to set share as public or not.
         :param share_group_id: text - ID of the share group to which the share
             should belong
+        :param scheduler_hints: dict - hints for the scheduler to place share
+            on most appropriate host e.g. keys are same_host for affinity and
+            different_host for anti-affinity
         :rtype: :class:`Share`
         """
         share_metadata = metadata if metadata is not None else dict()
+        scheduler_hints = (scheduler_hints if scheduler_hints is not None
+                           else dict())
         body = {
             'size': size,
             'snapshot_id': snapshot_id,
@@ -146,10 +157,11 @@ class ShareManager(base.ManagerWithFind):
             'description': description,
             'metadata': share_metadata,
             'share_proto': share_proto,
-            'share_network_id': common_base.getid(share_network),
-            'share_type': common_base.getid(share_type),
+            'share_network_id': base.getid(share_network),
+            'share_type': base.getid(share_type),
             'is_public': is_public,
             'availability_zone': availability_zone,
+            'scheduler_hints': scheduler_hints,
         }
         if share_group_id:
             body['share_group_id'] = share_group_id
@@ -286,7 +298,7 @@ class ShareManager(base.ManagerWithFind):
         :param share: either share object or text with its ID.
         """
         return self.api.client.post(
-            "/os-share-unmanage/%s/unmanage" % common_base.getid(share))
+            "/os-share-unmanage/%s/unmanage" % base.getid(share))
 
     @api_versions.wraps("2.7")  # noqa
     def unmanage(self, share):   # noqa
@@ -305,7 +317,7 @@ class ShareManager(base.ManagerWithFind):
         :param snapshot: either snapshot object or text with its ID.
         """
 
-        snapshot_id = common_base.getid(snapshot)
+        snapshot_id = base.getid(snapshot)
         info = {'snapshot_id': snapshot_id}
         return self._action('revert', share, info=info)
 
@@ -315,7 +327,7 @@ class ShareManager(base.ManagerWithFind):
         :param share: either share object or text with its ID.
         :rtype: :class:`Share`
         """
-        share_id = common_base.getid(share)
+        share_id = base.getid(share)
         return self._get("/shares/%s" % share_id, "share")
 
     def update(self, share, **kwargs):
@@ -328,7 +340,7 @@ class ShareManager(base.ManagerWithFind):
             return
 
         body = {'share': kwargs, }
-        share_id = common_base.getid(share)
+        share_id = base.getid(share)
         return self._update("/shares/%s" % share_id, body)
 
     @api_versions.wraps("1.0", "2.34")
@@ -337,13 +349,22 @@ class ShareManager(base.ManagerWithFind):
         """Get a list of all shares."""
         search_opts = search_opts or {}
         search_opts.pop("export_location", None)
+        search_opts.pop("is_soft_deleted", None)
         return self.do_list(detailed=detailed, search_opts=search_opts,
                             sort_key=sort_key, sort_dir=sort_dir)
 
-    @api_versions.wraps("2.35")   # noqa
+    @api_versions.wraps("2.35", "2.68")   # noqa
     def list(self, detailed=True, search_opts=None,   # noqa
              sort_key=None, sort_dir=None):
         """Get a list of all shares."""
+        search_opts.pop("is_soft_deleted", None)
+        return self.do_list(detailed=detailed, search_opts=search_opts,
+                            sort_key=sort_key, sort_dir=sort_dir)
+
+    @api_versions.wraps("2.69")  # noqa
+    def list(self, detailed=True, search_opts=None,  # noqa
+             sort_key=None, sort_dir=None):
+        """Get a list of all shares."""
         return self.do_list(detailed=detailed, search_opts=search_opts,
                             sort_key=sort_key, sort_dir=sort_dir)
 
@@ -367,6 +388,7 @@ class ShareManager(base.ManagerWithFind):
             - (('share_network_id', 'share_network'), text)
             - (('share_type_id', 'share_type'), text)
             - (('snapshot_id', 'snapshot'), text)
+            - ('is_soft_deleted', bool)
             Note, that member context will have restricted set of
             available search opts. For admin context filtering also available
             by each share attr from its Model. So, this list is not full for
@@ -388,6 +410,8 @@ class ShareManager(base.ManagerWithFind):
                     search_opts['sort_key'] = 'snapshot_id'
                 elif sort_key == 'share_network':
                     search_opts['sort_key'] = 'share_network_id'
+                elif sort_key == 'availability_zone':
+                    search_opts['sort_key'] = 'availability_zone_id'
             else:
                 raise ValueError('sort_key must be one of the following: %s.'
                                  % ', '.join(constants.SHARE_SORT_KEY_VALUES))
@@ -425,7 +449,7 @@ class ShareManager(base.ManagerWithFind):
         :param share_group_id: text - ID of the share group to which the share
             belongs
         """
-        url = "/shares/%s" % common_base.getid(share)
+        url = "/shares/%s" % base.getid(share)
         if share_group_id:
             url += "?share_group_id=%s" % share_group_id
         self._delete(url)
@@ -445,6 +469,22 @@ class ShareManager(base.ManagerWithFind):
     def force_delete(self, share):   # noqa
         return self._do_force_delete(share, "force_delete")
 
+    @api_versions.wraps("2.69")
+    def soft_delete(self, share):
+        """Soft delete a share - share will go to recycle bin.
+
+        :param share: either share object or text with its ID.
+        """
+        return self._action("soft_delete", share)
+
+    @api_versions.wraps("2.69")
+    def restore(self, share):
+        """Restore a share - share will restore from recycle bin.
+
+        :param share: either share object or text with its ID.
+        """
+        return self._action("restore", share)
+
     @staticmethod
     def _validate_common_name(access):
         if len(access) == 0 or len(access) > 64:
@@ -505,11 +545,11 @@ class ShareManager(base.ManagerWithFind):
             if access_type == 'ip':
                 try:
                     if enable_ipv6:
-                        ipaddress.ip_network(six.text_type(access))
+                        ipaddress.ip_network(str(access))
                     else:
-                        ipaddress.IPv4Network(six.text_type(access))
+                        ipaddress.IPv4Network(str(access))
                 except ValueError as error:
-                    raise exceptions.CommandError(six.text_type(error))
+                    raise exceptions.CommandError(str(error))
             elif access_type == 'user':
                 self._validate_username(access)
             elif access_type == 'cert':
@@ -624,7 +664,7 @@ class ShareManager(base.ManagerWithFind):
 
         :param share: either share object or text with its ID.
         """
-        return self._get("/shares/%s/metadata" % common_base.getid(share),
+        return self._get("/shares/%s/metadata" % base.getid(share),
                          "metadata")
 
     def set_metadata(self, share, metadata):
@@ -634,7 +674,7 @@ class ShareManager(base.ManagerWithFind):
         :param metadata: A list of keys to be set.
         """
         body = {'metadata': metadata}
-        return self._create("/shares/%s/metadata" % common_base.getid(share),
+        return self._create("/shares/%s/metadata" % base.getid(share),
                             body, "metadata")
 
     def delete_metadata(self, share, keys):
@@ -643,7 +683,7 @@ class ShareManager(base.ManagerWithFind):
         :param share: either share object or text with its ID.
         :param keys: A list of keys to be removed.
         """
-        share_id = common_base.getid(share)
+        share_id = base.getid(share)
         for key in keys:
             self._delete("/shares/%(share_id)s/metadata/%(key)s" % {
                 'share_id': share_id, 'key': key})
@@ -655,7 +695,7 @@ class ShareManager(base.ManagerWithFind):
         :param metadata: A list of keys to be updated.
         """
         body = {'metadata': metadata}
-        return self._update("/shares/%s/metadata" % common_base.getid(share),
+        return self._update("/shares/%s/metadata" % base.getid(share),
                             body)
 
     def _action(self, action, share, info=None, **kwargs):
@@ -668,7 +708,7 @@ class ShareManager(base.ManagerWithFind):
         """
         body = {action: info}
         self.run_hooks('modify_body_for_action', body, **kwargs)
-        url = '/shares/%s/action' % common_base.getid(share)
+        url = '/shares/%s/action' % base.getid(share)
         return self.api.client.post(url, body=body)
 
     def _do_reset_state(self, share, state, action_name):
@@ -687,22 +727,32 @@ class ShareManager(base.ManagerWithFind):
     def reset_state(self, share, state):  # noqa
         return self._do_reset_state(share, state, "reset_status")
 
-    def _do_extend(self, share, new_size, action_name):
+    def _do_extend(self, share, new_size, action_name, force=False):
         """Extend the size of the specified share.
 
         :param share: either share object or text with its ID.
         :param new_size: The desired size to extend share to.
+        :param force: if set to True, the scheduler's capacity decisions are
+                      not accounted for. Setting this parameter to True does
+                      not mean that the request will always succeed.
         """
-        return self._action(action_name, share, {"new_size": new_size})
+        req_body = {"new_size": new_size}
+        if force:
+            req_body['force'] = "true"
+        return self._action(action_name, share, req_body)
 
     @api_versions.wraps("1.0", "2.6")
     def extend(self, share, new_size):
         return self._do_extend(share, new_size, "os-extend")
 
-    @api_versions.wraps("2.7")  # noqa
+    @api_versions.wraps("2.7", "2.63")  # noqa
     def extend(self, share, new_size):  # noqa
         return self._do_extend(share, new_size, "extend")
 
+    @api_versions.wraps("2.64")  # noqa
+    def extend(self, share, new_size, force=False):  # noqa
+        return self._do_extend(share, new_size, "extend", force=force)
+
     def _do_shrink(self, share, new_size, action_name):
         """Shrink the size of the specified share.
 
@@ -725,7 +775,7 @@ class ShareManager(base.ManagerWithFind):
         :param share: either share object or text with its ID.
         """
         return self._list(
-            '/shares/%s/instances' % common_base.getid(share),
+            '/shares/%s/instances' % base.getid(share),
             'share_instances',
-            obj_class=share_instances.ShareInstance
+            manager=share_instances.ShareInstanceManager(self)
         )
